from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import max_langevin_fixed_point
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class MaxLangevinFixedPoint(AggregationPrimitive):
    """Largest fixed point of dynamics :math:argmax_x {h(x)=0}` estimated from
    polynomial :math:`h(x)`, which has been fitted to the deterministic
    dynamics of Langevin model

    .. math::
        \\dot(x)(t) = h(x(t)) + R \\mathcal(N)(0,1)

    as described by

        Friedrich et al. (2000): Physics Letters A 271, p. 217-222
        *Extracting model equations from experimental data*

    For short time-series this method is highly dependent on the parameters.

    Args:
        m (int) : Order of polynomial to fit for estimating fixed points of dynamics.
        r (float) : Number of quantils to use for averaging.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.max_langevin_fixed_point
    """

    name = "max_langevin_fixed_point"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, m, r):
        self.m = m
        self.r = r

    def get_function(self):
        def function(x):
            return max_langevin_fixed_point(x, m=self.m, r=self.r)

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import large_standard_deviation
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import BooleanNullable


class LargeStandardDeviation(AggregationPrimitive):
    """Boolean column denoting if the standard dev of x is higher than 'r'
    times the range = difference between max and min of x. Hence it checks if

    .. math::

        std(x) > r * (max(X)-min(X))

    According to a rule of the thumb, the standard deviation should be a forth
    of the range of the values.

    Args:
        r (float) : The percentage of the range to compare with.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.large_standard_deviation
    """

    name = "large_standard_deviation"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=BooleanNullable)
    stack_on_self = False

    def __init__(self, r):
        self.r = r

    def get_function(self):
        def function(x):
            return large_standard_deviation(x, r=self.r)

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import symmetry_looking
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import BooleanNullable


class SymmetryLooking(AggregationPrimitive):
    """Boolean column denoting if the distribution looks symmetric.

    Args:
        r (float) : Percentage of the range to compare with.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.symmetry_looking
    """

    name = "symmetry_looking"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=BooleanNullable)
    stack_on_self = False

    def __init__(self, r):
        self.r = r

    def get_function(self):
        def function(x):
            param = [{"r": self.r}]
            return symmetry_looking(x, param)[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import mean_change
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class MeanChange(AggregationPrimitive):
    """Returns the mean over the differences between subsequent time series
    values which is

    .. math::

        \\frac{1}{n} \\sum_{i=1,\\ldots, n-1}  x_{i+1} - x_{i}

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.mean_change
    """

    name = "mean_change"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def get_function(self):
        return mean_change



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import agg_autocorrelation
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class AggAutocorrelation(AggregationPrimitive):
    """Calculates the value of an aggregation function (e.g. the variance or
    the mean) over the autocorrelation for different lags.

    Args:
        f_agg (str) : Name of a numpy function (e.g. "mean", "var", "std",
            "median"), its the name of the aggregator function that is applied
            to the autocorrelations.
        maxlag (int) : Maximal number of lags to consider.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.agg_autocorrelation
    """

    name = "agg_autocorrelation"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, f_agg, maxlag):
        self.f_agg = f_agg
        self.maxlag = maxlag

    def get_function(self):
        def function(x):
            param = [{"f_agg": self.f_agg, "maxlag": self.maxlag}]
            return agg_autocorrelation(x, param=param)[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import sum_of_reoccurring_values
from woodwork.column_schema import ColumnSchema


class SumOfReoccurringValues(AggregationPrimitive):
    """Returns the sum of all values, that are present in the time series more
    than once.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.sum_of_reoccurring_values
    """

    name = "sum_of_reoccurring_values"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(semantic_tags={"numeric"})
    stack_on_self = False

    def get_function(self):
        return sum_of_reoccurring_values



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import index_mass_quantile
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class IndexMassQuantile(AggregationPrimitive):
    """Those apply features calculate the relative index i where q% of the mass
    of the time series x lie left of i. For example for q = 50% this feature
    calculator will return the mass center of the time series.

    Args:
        q (float) : Percentage of the mass of the time series.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.index_mass_quantile
    """

    name = "index_mass_quantile"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, q):
        self.q = q

    def get_function(self):
        def function(x):
            param = [{"q": self.q}]
            return index_mass_quantile(x, param=param)[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import linear_trend
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class LinearTrend(AggregationPrimitive):
    """Calculate a linear least-squares regression for the values of the time
    series versus the sequence from 0 to length of the time series minus one.
    This feature assumes the signal to be uniformly sampled. It will not use
    the time stamps to fit the model.

    Args:
        attr (str) : Controls which of the characteristics are returned.
            Possible extracted attributes are:
                ['pvalue', 'rvalue', 'intercept', 'slope', 'stderr'].

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.linear_trend
    """

    name = "linear_trend"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, attr):
        self.attr = attr

    def get_function(self):
        def function(x):
            param = [{"attr": self.attr}]
            return list(linear_trend(x, param))[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import count_above
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class CountAbove(AggregationPrimitive):
    """Returns the percentage of values in x that are higher than t

        Args:
            t (float) : value used as threshold

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.count_above
    """

    name = "count_above"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, t):
        self.t = t

    def get_function(self):
        def function(x):
            return count_above(x, self.t)

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import change_quantiles
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class ChangeQuantiles(AggregationPrimitive):
    """First fixes a corridor given by the quantiles ql and qh of the
    distribution of x. Then calculates the average, absolute value of
    consecutive changes of the series x inside this corridor.

    Args:
        ql (float) : The lower quantile of the corridor.
        qh (float) : The higher quantile of the corridor.
        isabs (bool) : Should the absolute differences be taken?
        f_agg (str) : The numpy function that aggregates (e.g. mean, var, std,
            median) the differences in the bin.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.change_quantiles
    """

    name = "change_quantiles"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, ql, qh, isabs, f_agg):
        self.ql = ql
        self.qh = qh
        self.isabs = isabs
        self.f_agg = f_agg

    def get_function(self):
        def function(x):
            return change_quantiles(
                x, ql=self.ql, qh=self.qh, isabs=self.isabs, f_agg=self.f_agg
            )

        return function



from featuretools_tsfresh_primitives import (
    SUPPORTED_PRIMITIVES,
    comprehensive_fc_parameters,
    primitives_from_fc_settings,
)


def test_primitives_from_fc_settings():
    fc_parameters = comprehensive_fc_parameters()

    for primitive in SUPPORTED_PRIMITIVES:
        inputs = fc_parameters[primitive.name] or [{}]
        parameters = {primitive.name: inputs}
        instances = primitives_from_fc_settings(parameters)

        assert len(instances) == len(inputs)
        items = zip(instances, inputs)

        for instance, inputs in items:
            assert instance.name == primitive.name

            for attribute, value in inputs.items():
                assert hasattr(instance, attribute)
                assert getattr(instance, attribute) == value



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import partial_autocorrelation
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class PartialAutocorrelation(AggregationPrimitive):
    """Calculates the value of the partial autocorrelation function at
    the given lag. The lag `k` partial autocorrelation of a time series
    :math:`\\lbrace x_t, t = 1 \\ldots T \\rbrace` equals the partial
    correlation of :math:`x_t` and :math:`x_{t-k}`, adjusted for the
    intermediate columns
    :math:`\\lbrace x_{t-1}, \\ldots, x_{t-k+1} \\rbrace` ([1]).
    Following [2], it can be defined as

    .. math::

        \\alpha_k = \\frac{ Cov(x_t, x_{t-k} | x_{t-1}, \\ldots, x_{t-k+1})}
        {\\sqrt{ Var(x_t | x_{t-1}, \\ldots, x_{t-k+1}) Var(x_{t-k} | x_{t-1}, \\ldots, x_{t-k+1} )}}

    with (a) :math:`x_t = f(x_{t-1}, \\ldots, x_{t-k+1})` and (b)
    :math:`x_{t-k} = f(x_{t-1}, \\ldots, x_{t-k+1})` being AR(k-1) models that
    can be fitted by OLS. Be aware that in (a), the regression is done on past
    values to predict :math:`x_t` whereas in (b), future values are used to
    calculate the past value :math:`x_{t-k}`. It is said in [1] that "for an
    AR(p), the partial autocorrelations [ :math:`\\alpha_k` ] will be nonzero
    for `k<=p` and zero for `k>p`." With this property, it is used to determine
    the lag of an AR-Process.

    .. rubric:: References

    |  [1] Box, G. E., Jenkins, G. M., Reinsel, G. C., & Ljung, G. M. (2015).
    |  Time series analysis: forecasting and control. John Wiley & Sons.
    |  [2] https://onlinecourses.science.psu.edu/stat510/node/62

    Args:
        lag (int) : The lag to be returned.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.partial_autocorrelation
    """

    name = "partial_autocorrelation"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, lag):
        self.lag = lag

    def get_function(self):
        def function(x):
            param = [{"lag": self.lag}]
            return partial_autocorrelation(x, param=param)[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import linear_trend_timewise
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Datetime, Double


class LinearTrendTimewise(AggregationPrimitive):
    """Calculate a linear least-squares regression for the values of the time series versus the sequence from 0 to length of the time series minus one.
    This feature uses the index of the time series to fit the model, which must be of a datetime dtype. The parameters control which of the characteristics are returned.
    Possible extracted attributes are "pvalue", "rvalue", "intercept", "slope", "stderr", see the documentation of linregress for more information.

    Args:
        attr (str) : Controls which of the characteristics are returned.
            Possible extracted attributes are: pvalue, rvalue, intercept, slope, or stderr.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.linear_trend_timewise
    """

    name = "linear_trend_timewise"
    input_types = [
        ColumnSchema(semantic_tags={"numeric"}),
        ColumnSchema(semantic_tags={"time_index"}, logical_type=Datetime),
    ]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, attr):
        self.attr = attr

    def get_function(self):
        def function(numeric, time):
            numeric.index = time.values
            param = [{"attr": self.attr}]
            values = linear_trend_timewise(numeric, param)
            return values[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import sample_entropy
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class SampleEntropy(AggregationPrimitive):
    """Calculate and return sample entropy of x.

    .. rubric:: References

    |  [1] http://en.wikipedia.org/wiki/Sample_Entropy
    |  [2] https://www.ncbi.nlm.nih.gov/pubmed/10843903?dopt=Abstract

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.sample_entropy
    """

    name = "sample_entropy"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def get_function(self):
        return sample_entropy



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import range_count
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import IntegerNullable


class RangeCount(AggregationPrimitive):
    """Count observed values within the interval [min, max).

    Args:
        min (float) : The inclusive lower bound of the range.
        max (float) : The exclusive upper bound of the range.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.range_count
    """

    name = "range_count"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=IntegerNullable, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, min, max):
        self.min = min
        self.max = max

    def get_function(self):
        def function(x):
            return range_count(x, min=self.min, max=self.max)

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import has_duplicate_min
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import BooleanNullable


class HasDuplicateMin(AggregationPrimitive):
    """Checks if the minimal value of x is observed more than once.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.has_duplicate_min
    """

    name = "has_duplicate_min"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=BooleanNullable)
    stack_on_self = False

    def get_function(self):
        return has_duplicate_min



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import (
    mean_second_derivative_central,
)
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class MeanSecondDerivativeCentral(AggregationPrimitive):
    """Returns the mean value of a central approximation
    of the second derivative

    .. math::

        \\frac{1}{n} \\sum_{i=1,\\ldots, n-1}  \\frac{1}{2} (x_{i+2} - 2 \\cdot x_{i+1} + x_i)

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.mean_second_derivative_central
    """

    name = "mean_second_derivative_central"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def get_function(self):
        return mean_second_derivative_central



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import fft_aggregated
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class FftAggregated(AggregationPrimitive):
    """Returns the spectral centroid (mean), variance, skew, and kurtosis of
    the absolute fourier transform spectrum.

    Args:
        aggtype (str) : Controls which aggregation is returned. Possible values
            are: ["centroid", "variance", "skew", "kurtosis"]

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.fft_aggregated
    """

    name = "fft_aggregated"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, aggtype):
        self.aggtype = aggtype

    def get_function(self):
        def function(x):
            param = [{"aggtype": self.aggtype}]
            return list(fft_aggregated(x, param=param))[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import ratio_beyond_r_sigma
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class RatioBeyondRSigma(AggregationPrimitive):
    """Ratio of values that are more than r*std(x) (so r sigma) away from the
    mean of x.

    Args:
        r (float) : Weight of sigma.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.ratio_beyond_r_sigma
    """

    name = "ratio_beyond_r_sigma"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, r):
        self.r = r

    def get_function(self):
        def function(x):
            return ratio_beyond_r_sigma(x, r=self.r)

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import skewness
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class Skewness(AggregationPrimitive):
    """Returns the sample skewness of x (calculated with the adjusted
    Fisher-Pearson standardized moment coefficient G1).

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.skewness
    """

    name = "skewness"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def get_function(self):
        return skewness



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import (
    ratio_value_number_to_time_series_length,
)
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class RatioValueNumberToTimeSeriesLength(AggregationPrimitive):
    """Returns a factor which is 1 if all values in the time series occur only
    once, and below one if this is not the case. In principle, it just returns

        # unique values / # values

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.ratio_value_number_to_time_series_length
    """

    name = "ratio_value_number_to_time_series_length"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def get_function(self):
        return ratio_value_number_to_time_series_length



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import number_crossing_m
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import IntegerNullable


class NumberCrossingM(AggregationPrimitive):
    """Calculates the number of crossings of x on m. A crossing is defined as
    two sequential values where the first value is lower than m and the next is
    greater, or vice-versa. If you set m to zero, you will get the number of
    zero crossings.

    Args:
        m (float) : The threshold for the crossing.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.number_crossing_m
    """

    name = "number_crossing_m"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=IntegerNullable, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, m):
        self.m = m

    def get_function(self):
        def function(x):
            return number_crossing_m(x, m=self.m)

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import autocorrelation
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class Autocorrelation(AggregationPrimitive):
    """Calculates the autocorrelation of the specified lag.

    Args:
        lag (int) : The lag.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.autocorrelation
    """

    name = "autocorrelation"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, lag):
        self.lag = lag

    def get_function(self):
        def function(x):
            return autocorrelation(x, lag=self.lag)

        return function



from os import path

from setuptools import find_packages, setup

dirname = path.abspath(path.dirname(__file__))
with open(path.join(dirname, 'README.md')) as f:
    long_description = f.read()

setup(
    name='featuretools_tsfresh_primitives',
    version='1.0.2',
    author='Feature Labs, Inc.',
    author_email='support@featurelabs.com',
    license='MIT',
    description='TSFresh primitives for featuretools',
    url='https://github.com/alteryx/featuretools-tsfresh-primitives/',
    classifiers=[
         'Development Status :: 3 - Alpha',
         'Intended Audience :: Developers',
         'Programming Language :: Python :: 3',
         'Programming Language :: Python :: 3.7',
         'Programming Language :: Python :: 3.8',
         'Programming Language :: Python :: 3.9',
    ],
    packages=find_packages(),
    install_requires=open('requirements.txt').readlines(),
    tests_require=open('test-requirements.txt').readlines(),
    python_requires='>=3.7, <4',
    include_package_data=True,
    entry_points={
        'featuretools_plugin': [
            'tsfresh = featuretools_tsfresh_primitives.__init__',
        ],
    },
    long_description=long_description,
    long_description_content_type='text/markdown'
)



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import cwt_coefficients
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class CwtCoefficients(AggregationPrimitive):
    """Calculates a Continuous wavelet transform for the Ricker wavelet, also
    known as the "Mexican hat wavelet" which is defined by

    .. math::
        \\frac{2}{\\sqrt{3a} \\pi^{\\frac{1}{4}}} (1 - \\frac{x^2}{a^2}) exp(-\\frac{x^2}{2a^2})

    where :math:`a` is the width parameter of the wavelet function.

    This feature calculator takes three different parameter: widths, coeff and
    w. The feature calculater takes all the different widths arrays and then
    calculates the cwt one time for each different width array. Then the values
    for the different coefficient for coeff and width w are returned.

    Args:
        widths (list[int]) : The different widths arrays.
        coeff (int) : The different coefficient to return.
        w (int) : The different width to return.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.cwt_coefficients
    """

    name = "cwt_coefficients"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, widths, coeff, w):
        self.widths = tuple(widths)
        self.coeff = coeff
        self.w = w

    def get_function(self):
        def function(x):
            param = [{"widths": self.widths, "coeff": self.coeff, "w": self.w}]
            return list(cwt_coefficients(x, param=param))[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import energy_ratio_by_chunks
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class EnergyRatioByChunks(AggregationPrimitive):
    """Calculates the sum of squares of chunk i out of N chunks expressed as a
    ratio with the sum of squares over the whole series.

    Args:
        num_segments (int) : Number of segments to divide the series into.
        segment_focus (int) : Segment number (starting at zero) to return a feature on.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.energy_ratio_by_chunks
    """

    name = "energy_ratio_by_chunks"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, num_segments, segment_focus):
        self.num_segments = num_segments
        self.segment_focus = segment_focus

    def get_function(self):
        def function(x):
            param = [
                {"num_segments": self.num_segments, "segment_focus": self.segment_focus}
            ]
            return energy_ratio_by_chunks(x, param)[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.feature_calculators import ar_coefficient
from woodwork.column_schema import ColumnSchema
from woodwork.logical_types import Double


class ArCoefficient(AggregationPrimitive):
    """This feature calculator fits the unconditional maximum likelihood of an
    autoregressive AR(k) process.

    Args:
        coeff (int) : Index of returned coefficient.
        k (int) : Maximum lag of the process.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.ar_coefficient
    """

    name = "ar_coefficient"
    input_types = [ColumnSchema(semantic_tags={"numeric"})]
    return_type = ColumnSchema(logical_type=Double, semantic_tags={"numeric"})
    stack_on_self = False

    def __init__(self, coeff, k):
        self.coeff = coeff
        self.k = k

    def get_function(self):
        def function(x):
            param = [{"coeff": self.coeff, "k": self.k}]
            return ar_coefficient(x, param=param)[0][1]

        return function
